type GraphStatistics = {
  topological: TopologicalStats;
  nodes: NodeOrEdgeStats;
  edges: NodeOrEdgeStats;
};

type NodeOrEdgeStats = {
  count: number;
  labels: string[];
  types: {
    [label: string]: {
      count: number;
      avgDegreeIn?: number;
      avgDegreeOut?: number;
      attributes: {
        [id: string]: AttributeStats<AttributeType>;
      };
    };
  };
};

type AttributeStats<T extends AttributeType> = {
  attributeType: T;
  statistics: AttributeTypeStats<T>;
};

type AttributeTypeStats<T extends AttributeType> = T extends 'string'
  ? CategoricalStats
  : T extends 'boolean'
    ? BooleanStats
    : T extends 'number'
      ? NumericalStats
      : T extends 'date' | 'time' | 'datetime' | 'timestamp'
        ? TemporalStats
        : T extends 'array'
          ? ArrayStats
          : T extends 'object'
            ? ObjectStats
            : never;

type AttributeType = 'string' | 'boolean' | 'number' | 'array' | 'object' | TemporalType;

type TemporalType = 'date' | 'time' | 'datetime' | 'timestamp';
// Date: Date in the YYYY-MM-DD format (ISO 8601 syntax) (e.g., 2021-09-28)
// Time: Time in the hh:mm:ss format for the time of day, time since an event, or time interval between events (e.g., 12:00:59)
// Datetime: Date and time together in the YYYY-MM-DD hh:mm:ss format (e.g., 2021-09-28 12:00:59)
// Timestamp: Number of seconds that have elapsed since midnight (00:00:00 UTC), 1st January (Unix time) (e.g., 1632855600)

type TopologicalStats = {
  density: number;
  self_loops: number;
};

type NumericalStats = {
  min: number;
  max: number;
  average: number;
  count: number;
};

type BooleanStats = {
  true: number;
  false: number;
};

type CategoricalStats = {
  uniqueItems: number;
  values: string[];
  mode: string;
  count: number;
};

type TemporalStats = {
  min: number;
  max: number;
  range: number;
};

type ArrayStats = {
  length: number;
  count: number;
};

type ObjectStats = {
  length: number;
};

export type {
  GraphStatistics,
  AttributeStats,
  NumericalStats,
  CategoricalStats,
  BooleanStats,
  TemporalStats,
  AttributeType,
  AttributeTypeStats,
  ArrayStats,
  ObjectStats,
};
